import sys
sys.path.append("../../pypdevs/src/")

from DEVS import *
from infinity import INFINITY
from util import *

class GeneratorState:
	def __init__(self, name=""):
		self.name = name

class QueueState:
	def __init__(self, name="", message=None):
		self.name = name
		self.message = message

class CollectorState:
	def __init__(self, name=""):
		self.name = name

class InputMessage:
	def __init__(self, processing_time=None):
		self.processing_time = processing_time

class OutputMessage:
	def __init__(self, processing_time=None):
		self.processing_time = processing_time

class Generator(AtomicDEVS):
	def __init__(self, name="Generator"):
		AtomicDEVS.__init__(self, name)

		self.state = GeneratorState(name="generating")
		self.my_ports = {"out": self.addOutPort("out")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return 50

	def outputFnc(self):
		if self.state.name == "generating":
			return {self.my_ports[k]: v for k, v in {'out': [OutputMessage(generate_random(0.5, 5.0))]}.iteritems()}

	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
		def action_int_generating_to_generating():
			return {}
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return GeneratorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

	def confTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.confTransition(self, my_inputs)

class Queue(AtomicDEVS):
	def __init__(self, name="Queue", speed=None):
		AtomicDEVS.__init__(self, name)
		self.speed = speed if speed is not None else 1
		self.state = QueueState(name="idle")
		self.my_ports = {"out": self.addOutPort("out"), "in": self.addInPort("in")}

	def timeAdvance(self):
		if self.state.name == "idle":
			return INFINITY
		if self.state.name == "processing":
			return self.state.message.processing_time / self.speed

	def outputFnc(self):
		if self.state.name == "idle":
			return {}
		if self.state.name == "processing":
			return {self.my_ports[k]: v for k, v in {'out': [OutputMessage(self.state.message.processing_time)]}.iteritems()}

	def intTransition(self):
		def cond_int_processing_to_idle():
			return True
		def action_int_processing_to_idle():
			return {}
		if self.state.name == "processing" and cond_int_processing_to_idle():
			return QueueState(name="idle", **action_int_processing_to_idle())

		else:
			return AtomicDEVS.intTransition(self)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_idle_to_processing():
			return True
		def action_ext_idle_to_processing():
			return {"message": inputs['in'][0]}
		if self.state.name == "idle" and cond_ext_idle_to_processing():
			return QueueState(name="processing", **action_ext_idle_to_processing())

		else:
			return AtomicDEVS.extTransition(self, my_inputs)

	def confTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.confTransition(self, my_inputs)

class Collector(AtomicDEVS):
	def __init__(self, name="Collector"):
		AtomicDEVS.__init__(self, name)

		self.state = CollectorState(name="waiting")
		self.my_ports = {"in": self.addInPort("in")}

	def timeAdvance(self):
		if self.state.name == "waiting":
			return INFINITY

	def outputFnc(self):
		if self.state.name == "waiting":
			return {}

	def intTransition(self):
		return AtomicDEVS.intTransition(self)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_waiting_to_waiting():
			return True
		def action_ext_waiting_to_waiting():
			return {}
		if self.state.name == "waiting" and cond_ext_waiting_to_waiting():
			return CollectorState(name="waiting", **action_ext_waiting_to_waiting())

		else:
			return AtomicDEVS.extTransition(self, my_inputs)

	def confTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.confTransition(self, my_inputs)

class Root(CoupledDEVS):
	def __init__(self, name="Root"):
		CoupledDEVS.__init__(self, name)
		self.my_ports = {}
		self.submodels = {}
		self.submodels["g"] = self.addSubModel(Generator(name="g"))
		self.submodels["q1"] = self.addSubModel(Queue(name="q1", speed=2))
		self.submodels["q2"] = self.addSubModel(Queue(name="q2", speed=0.5))
		self.submodels["c"] = self.addSubModel(Collector(name="c"))

		def transfer_g_out_to_in(event):
			return InputMessage(event.processing_time)

		self.connectPorts(self.submodels["g"].my_ports["out"], self.submodels["q1"].my_ports["in"], transfer_g_out_to_in)

		def transfer_q1_out_to_in(event):
			return InputMessage(event.processing_time)

		self.connectPorts(self.submodels["q1"].my_ports["out"], self.submodels["q2"].my_ports["in"], transfer_q1_out_to_in)

		def transfer_q2_out_to_in(event):
			return InputMessage(event.processing_time)

		self.connectPorts(self.submodels["q2"].my_ports["out"], self.submodels["c"].my_ports["in"], transfer_q2_out_to_in)


