import sys
sys.path.append("../../pypdevs/src/")

from DEVS import *
from infinity import INFINITY
from util import *

class TrafficLightMode:
	def __init__(self, name="", time=None):
		self.name = name
		self.time = time

class PolicemanMode:
	def __init__(self, name=""):
		self.name = name

class TrafficLight(AtomicDEVS):
	def __init__(self, name="TrafficLight"):
		AtomicDEVS.__init__(self, name)

		self.state = TrafficLightMode(name="red", time=0)
		self.my_ports = {"OBSERVED": self.addOutPort("OBSERVED"), "INTERRUPT": self.addInPort("INTERRUPT")}

	def timeAdvance(self):
		if self.state.name == "red":
			return 60
		if self.state.name == "green":
			return 50
		if self.state.name == "yellow":
			return 10
		if self.state.name == "manual":
			return INFINITY

	def outputFnc(self):
		if self.state.name == "red":
			return {self.my_ports[k]: v for k, v in {"OBSERVED": ["grey"]}.iteritems()}
		if self.state.name == "green":
			return {self.my_ports[k]: v for k, v in {"OBSERVED": ["yellow"]}.iteritems()}
		if self.state.name == "yellow":
			return {self.my_ports[k]: v for k, v in {"OBSERVED": ["grey"]}.iteritems()}
		if self.state.name == "manual":
			return {}

	def intTransition(self):
		def cond_int_red_to_green():
			return True
		def action_int_red_to_green():
			return {'time': self.state.time + self.timeAdvance()}
		if self.state.name == "red" and cond_int_red_to_green():
			return TrafficLightMode(name="green", **action_int_red_to_green())

		def cond_int_green_to_yellow():
			return True
		def action_int_green_to_yellow():
			return {'time': self.state.time + self.timeAdvance()}
		if self.state.name == "green" and cond_int_green_to_yellow():
			return TrafficLightMode(name="yellow", **action_int_green_to_yellow())

		def cond_int_yellow_to_red():
			return True
		def action_int_yellow_to_red():
			return {'time': self.state.time + self.timeAdvance()}
		if self.state.name == "yellow" and cond_int_yellow_to_red():
			return TrafficLightMode(name="red", **action_int_yellow_to_red())

		else:
			return AtomicDEVS.intTransition(self)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_red_to_manual():
			return inputs['INTERRUPT'][0] == 'toManual'
		def action_ext_red_to_manual():
			return {'time': self.state.time + self.elapsed}
		if self.state.name == "red" and cond_ext_red_to_manual():
			return TrafficLightMode(name="manual", **action_ext_red_to_manual())

		def cond_ext_green_to_manual():
			return inputs['INTERRUPT'][0] == 'toManual'
		def action_ext_green_to_manual():
			return {'time': self.state.time + self.elapsed}
		if self.state.name == "green" and cond_ext_green_to_manual():
			return TrafficLightMode(name="manual", **action_ext_green_to_manual())

		def cond_ext_yellow_to_manual():
			return inputs['INTERRUPT'][0] == 'toManual'
		def action_ext_yellow_to_manual():
			return {'time': self.state.time + self.elapsed}
		if self.state.name == "yellow" and cond_ext_yellow_to_manual():
			return TrafficLightMode(name="manual", **action_ext_yellow_to_manual())

		def cond_ext_manual_to_red():
			return inputs['INTERRUPT'][0] == 'toAutonomous'
		def action_ext_manual_to_red():
			return {'time': self.state.time + self.elapsed}
		if self.state.name == "manual" and cond_ext_manual_to_red():
			return TrafficLightMode(name="red", **action_ext_manual_to_red())

		else:
			return AtomicDEVS.extTransition(self, my_inputs)

	def confTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.confTransition(self, my_inputs)

class Policeman(AtomicDEVS):
	def __init__(self, name="Policeman"):
		AtomicDEVS.__init__(self, name)

		self.state = PolicemanMode(name="idle")
		self.my_ports = {"OUT": self.addOutPort("OUT")}

	def timeAdvance(self):
		if self.state.name == "idle":
			return 200
		if self.state.name == "working":
			return 100

	def outputFnc(self):
		if self.state.name == "idle":
			return {self.my_ports[k]: v for k, v in {"OUT": ["toManual"]}.iteritems()}
		if self.state.name == "working":
			return {self.my_ports[k]: v for k, v in {"OUT": ["toAutonomous"]}.iteritems()}

	def intTransition(self):
		def cond_int_idle_to_working():
			return True
		def action_int_idle_to_working():
			return {}
		if self.state.name == "idle" and cond_int_idle_to_working():
			return PolicemanMode(name="working", **action_int_idle_to_working())

		def cond_int_working_to_idle():
			return True
		def action_int_working_to_idle():
			return {}
		if self.state.name == "working" and cond_int_working_to_idle():
			return PolicemanMode(name="idle", **action_int_working_to_idle())

		else:
			return AtomicDEVS.intTransition(self)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

	def confTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.confTransition(self, my_inputs)

class Root(CoupledDEVS):
	def __init__(self, name="Root"):
		CoupledDEVS.__init__(self, "Root")
		self.my_ports = {}
		self.submodels = {}
		self.submodels["policeman"] = self.addSubModel(Policeman(name="policeman"))
		self.submodels["trafficLight"] = self.addSubModel(TrafficLight(name="trafficLight"))

		self.connectPorts(self.submodels["policeman"].my_ports["OUT"], self.submodels["trafficLight"].my_ports["INTERRUPT"])

