'''
Created on 14-okt.-2014

@author: Simon
'''

import sys
sys.path.append("")
sys.path.append("../pypdevs/src/")

import logging, json
from collections import defaultdict
from utils import Utilities as utils
from ptcal import PyTCoreAbstractionLayer
from exported_to_pypdevs.experiment import *
from python_runtime.statecharts_core import Event

class PyTCoreAbstractionLayerWithParallelDEVS(PyTCoreAbstractionLayer):
    def __init__(self, aswCommTools, mtwid):
        PyTCoreAbstractionLayer.__init__(self, aswCommTools, mtwid)
        self.model = Root()
        self.controller = Controller(self.model)
        self.controller.start()
        self.controller.addInput(Event("trace", "request", ['trace.txt']))
        self.reply_port = self.controller.addOutputListener("reply")
        self._m = None
        self._m_atompm = None
        self.messages = []
        self.breakpoints = []
        self.incoming = defaultdict(set)
        self.outgoing = defaultdict(set)
    
    def __calc_in_out_rel(self, rel_type):
        print rel_type
        print self.type_map_keys[rel_type]
        for key in self.type_map_keys[rel_type]:
            for e in self._m_atompm['edges']:
                if e['dest'] == key:
                    self.incoming[key].add(e['src'])
                    self.outgoing[e['src']].add(key)
                if e['src'] == key:
                    self.outgoing[key].add(e['dest'])
                    self.incoming[e['dest']].add(key)

    def loadModel(self, m, mms, sn):
        assert self._m == None, 'ptcal.loadModel() should only be called once'
        if m.__class__ != {}.__class__ :
            m = json.loads(m)
        self._m_atompm = m
        model = {}
        if len(m['nodes']) == 0:
            return model
        ''' look for root, but also build a type_map '''
        self.type_map = defaultdict(list)
        self.type_map_keys = defaultdict(list)
        for key, node in m['nodes'].iteritems():
            self.type_map[node["$type"]].append(node)
            self.type_map_keys[node["$type"]].append(key)
            if node["$type"] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/CoupledDEVS' and node['name']['value'] == 'Root':
                model['Root'] = node
                node['$loc'] = 'Root'
            elif node['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/GlobalBreakpoint':
                self.breakpoints.append(key)
                condition = '\n'.join(map(lambda l: '\t%s' % l, node['condition']['value'].split("\n")))
                bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n%s' % condition
                self.controller.addInput(Event("add_breakpoint", "request", [key, bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))
            node['$key'] = key
        ''' find all incoming and outgoing elements '''
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/submodels')
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/states')
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/state')
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/ports')
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Debug/target')
        self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/injectLocation')
        model_queue = [model['Root']]
        while len(model_queue) > 0:
            curr_model = model_queue.pop()
            for o_rel_key in filter(lambda x: m['nodes'][x]['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/submodels', self.outgoing[curr_model['$key']]):
                submodel = m['nodes'][next(iter(self.outgoing[o_rel_key]))]
                submodel['$loc'] = curr_model['$loc'] + '.' + submodel['name']['value']
                model[submodel['$loc']] = submodel
                model_queue.append(submodel)
            for o_rel_key in filter(lambda x: m['nodes'][x]['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/states', self.outgoing[curr_model['$key']]):
                state = m['nodes'][next(iter(self.outgoing[o_rel_key]))]
                state['$loc'] = curr_model['$loc'] + '.' + state['name']['value']
                model[state['$loc']] = state
                model.setdefault(curr_model['$loc'] + '.' + '$states', []).append(state)
            for o_rel_key in filter(lambda x: m['nodes'][x]['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/ports', self.outgoing[curr_model['$key']]):
                port = m['nodes'][next(iter(self.outgoing[o_rel_key]))]
                port['$loc'] = curr_model['$loc'] + '.' + port['name']['value']
                model[port['$loc']] = port
            if curr_model['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/AtomicDEVS':
                for o_rel_key in filter(lambda x: m['nodes'][x]['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/state', self.outgoing[curr_model['$key']]):
                    state = m['nodes'][next(iter(self.outgoing[o_rel_key]))]
                    state['$loc'] = curr_model['$loc'] + '.' + '$state'
                    model[state['$loc']] = state
        if '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/Breakpoint' in self.type_map_keys:
            for key in self.type_map_keys['/Formalisms/ParallelDEVS/ParallelDEVS_Debug/Breakpoint']:
                node = m['nodes'][key]
                self.breakpoints.append(key)
                condition = '\n'.join(map(lambda l: '\t%s' % l, node['condition']['value'].split("\n"))) if len(node['condition']['value']) > 0 else 'return True'
                state_id = next(iter(self.outgoing[next(iter(self.outgoing[key]))]))
                state = self._m_atompm['nodes'][state_id]
                bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n\tif len(filter(lambda x: x.getModelFullName() == "%s" and x.state.name == "%s", transitioned)) == 0:\n\t\treturn False\n%s' % (state['$loc'][:state['$loc'].rfind(".")], state['name']['value'], condition)
                self.controller.addInput(Event("add_breakpoint", "request", [key, bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))
        if '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance' in self.type_map_keys:
            for key in self.type_map_keys['/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance']:
                node = m['nodes'][key]
                self.messages.append(key)
                port_id = next(iter(self.outgoing[next(iter(self.outgoing[key]))]))
                port = self._m_atompm['nodes'][port_id]
                self.controller.addInput(Event("inject", "request", [{"port": port['$loc'], "event": eval(node['event_type']['value'] + '(' + ','.join(map(lambda x: '%s = %s' % (x['name'], x['val']), node['attribute_values']['value'])) + ')')}]))
        model['$simulation'] = self.type_map['/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/SimulationInstance'][0]
        self._m = model
        utils.setTimeout(0.1, self._simulate)
        
    def loadTransforms(self, fnames):
        pass

    def simulate(self):
        self._requestUnhighlight()
        params = [{"termination_condition": termination_condition}]
        self.controller.addInput(Event("simulate", "request", params))

    def simulatert(self, scale_factor):
        self._requestUnhighlight()
        params = [{"termination_condition": termination_condition, "realtime_scale": scale_factor}]
        self.controller.addInput(Event("realtime", "request", params))

    def bigstep(self):
        self._requestUnhighlight()
        params = [{"termination_condition": termination_condition}]
        self.controller.addInput(Event("big_step", "request", params))

    def backstep(self):
        print 'in backstep'
        self._requestUnhighlight()
        self.controller.addInput(Event("backwards_step", "request", []))
    
    def smallstep(self):
        self._requestUnhighlight()
        params = [{"termination_condition": termination_condition}]
        self.controller.addInput(Event("small_step", "request", params))

    def pause(self):
        self._requestUnhighlight()
        self.controller.addInput(Event("pause", "request", []))

    def reset(self):
        self._requestUnhighlight()
        self.controller.addInput(Event("reset", "request", []))

    def showtrace(self):
        with open('trace.txt') as f:
            self._aswCommTools['httpReq']('PUT','/GET/console', {'text': f.read()})

    def godevent(self, data):
        reqs = []
        value = eval(data['value'])
        if data['attribute'] == 'name':
            reqs.append(self._changeAttr(data['model'] + '.' + value, 'current', True))
            for locstate in self._m[data['model'] + '.' + '$states']:
                if locstate['name']['value'] != value:
                    reqs.append(self._changeAttr(data['model'] + '.' + locstate['name']['value'], 'current', False))
        else:
            for locstate in self._m[data['model'] + '.' + '$states']:
                if locstate['current']['value'] == True:
                    attr_vals = self._m_atompm['nodes'][self._m[data['model'] + '.$state']['$key']]['attribute_values']['value']
                    attr_val = filter(lambda x: x['name'] == data['attribute'], attr_vals)
                    json_value = None
                    try:
                        json_value = json.dumps(value)
                    except TypeError:
                        json_value = str(value)
                    if len(attr_val) == 0:
                        attr_vals.append({'name': data['attribute'], 'val': json_value})
                    else:
                        attr_val[0]['val'] = json_value
                    reqs.append(self._changeAttr(data['model'] + '.$state', 'attribute_values', attr_vals))
        if reqs:
            self._aswCommTools['httpReq']('POST', '/batchEdit', reqs)
        self.controller.addInput(Event("god_event", "request", [{"model": data['model'], "attribute": data['attribute'], "value": value}]))

    def inject(self):
        for msg_id in self.type_map_keys['/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance']:
            msg = self._m_atompm['nodes'][msg_id]
            if not 'injected' in msg:
                if msg_id in self.outgoing:
                    conn_id = next(iter(self.outgoing[msg_id]))
                    port_id = next(iter(self.outgoing[conn_id]))
                    conn = self._m_atompm['nodes'][conn_id]
                    self.controller.addInput(Event("inject", "request", [{"time": conn['time']['value'] if conn['time']['value'] != -1 else self._m_atompm['nodes'][self._m['$simulation']['$key']]['simulation_time']['value'], "port": self._m_atompm['nodes'][port_id]['$loc'], "event": eval(msg['event_type']['value'] + '(' + ','.join(map(lambda x: '%s = %s' % (x['name'], x['val']), msg['attribute_values']['value'])) + ')')}]))
                    msg['injected'] = True
                    msg['injection_time'] = conn['time']['value']
        
    def onchangelog(self, c):
        bp_connected = None
        message_connected = None
        for cl in c['changelog']:
            if cl["op"] == 'CHATTR':
                self._m_atompm['nodes'][cl['id']][cl['attr']]['value'] = cl['new_val']
            if cl["op"] == 'RMNODE' and cl['id'] in self.messages:
                self.messages.remove(cl['id'])
            if cl["op"] == 'MKNODE':
                node = json.loads(cl['node'])
                self._m_atompm['nodes'][str(cl['id'])] = node
                self.type_map[node['$type']].append(node)
                self.type_map_keys[node['$type']].append(str(cl['id']))
            if cl["op"] == 'MKNODE' and json.loads(cl['node'])["$type"] == '/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance':
                self.messages.append(str(cl["id"]))
            elif cl["op"] == 'MKNODE' and json.loads(cl['node'])["$type"] == '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/GlobalBreakpoint':
                node = json.loads(cl['node'])
                self.breakpoints.append(str(cl["id"]))
                condition = '\treturn False'
                bp_f = 'def breakpoint(time, model, transitioned):\n%s' % condition
                self.controller.addInput(Event("add_breakpoint", "request", [str(cl['id']), bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))
            elif cl["op"] == 'MKNODE' and json.loads(cl['node'])["$type"] == '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/Breakpoint':
                self.breakpoints.append(str(cl["id"]))
            elif cl["op"] == 'CHATTR' and cl["id"] in self.breakpoints:
                if cl["attr"] == "condition":
                    self.controller.addInput(Event("del_breakpoint", "request", [cl['id']]))
                    node = self._m_atompm['nodes'][cl["id"]]
                    node['condition']['value'] = cl['new_val']
                    condition = '\n'.join(map(lambda l: '\t%s' % l, cl['new_val'].split("\n")))
                    if node['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/GlobalBreakpoint':
                        bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n%s' % condition
                    else:
                        state_id = next(iter(self.outgoing[next(iter(self.outgoing[cl["id"]]))]))
                        state = self._m_atompm['nodes'][state_id]
                        bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n\tif len(filter(lambda x: x.getModelFullName() == "%s" and x.state.name == "%s", transitioned)) == 0:\n\t\treturn False\n%s' % (state['$loc'][:state['$loc'].rfind(".")], state['name']['value'], condition)
                    self.controller.addInput(Event("add_breakpoint", "request", [cl['id'], bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))
                elif cl["attr"] == "enabled":
                    node = self._m_atompm['nodes'][cl["id"]]
                    node['enabled']['value'] = cl['new_val']
                    self.controller.addInput(Event("toggle_breakpoint", "request", [cl["id"], node['enabled']['value']]))
                elif cl["attr"] == "disable_after_trigger":
                    self.controller.addInput(Event("del_breakpoint", "request", [cl['id']]))
                    node = self._m_atompm['nodes'][cl["id"]]
                    node['disable_after_trigger']['value'] = cl['new_val']
                    condition = '\n'.join(map(lambda l: '\t%s' % l, node['condition']['value'].split('\n')))
                    if node['$type'] == '/Formalisms/ParallelDEVS/ParallelDEVS_Debug/GlobalBreakpoint':
                        bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n%s' % condition
                    else:
                        state_id = next(iter(self.outgoing[next(iter(self.outgoing[cl["id"]]))]))
                        state = self._m_atompm['nodes'][state_id]
                        bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n\tif len(filter(lambda x: x.getModelFullName() == "%s" and x.state.name == "%s", transitioned)) == 0:\n\t\treturn False\n%s' % (state['$loc'][:state['$loc'].rfind(".")], state['name']['value'], condition)
                    self.controller.addInput(Event("add_breakpoint", "request", [cl['id'], bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))
            elif cl["op"] == 'RMNODE' and cl["id"] in self.breakpoints:
                self.breakpoints.remove(cl["id"])
                self.controller.addInput(Event("del_breakpoint", "request", [cl["id"]]))
            elif cl["op"] == 'MKEDGE':
                self._m_atompm['edges'].append({'src': cl['id1'], 'dest': cl['id2']})
                self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Debug/target')
                self.__calc_in_out_rel('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/injectLocation')
                if cl['id1'] in self.breakpoints:
                    bp_connected = cl['id1']
                elif cl['id1'] in self.messages:
                    message_connected = cl['id1']
        if bp_connected:
            state_id = next(iter(self.outgoing[next(iter(self.outgoing[bp_connected]))]))
            node = self._m_atompm['nodes'][bp_connected]
            condition = '\n'.join(map(lambda l: '\t%s' % l, node['condition']['value'].split("\n"))) if len(node['condition']['value']) > 0 else 'return True'
            bp_f = 'def breakpoint(time, model, transitioned):\n\ttime = time[0]\n\tif not "%s" in transitioned:\n\t\treturn False\n%s' % (self._m_atompm['nodes'][state_id]['$loc'], condition)
            self.controller.addInput(Event("add_breakpoint", "request", [bp_connected, bp_f, node['enabled']['value'], node['disable_after_trigger']['value']]))

    def _simulate(self):
        stateChange = None
        while True:
            stateChange = self.reply_port.fetch(0)
            if stateChange is None:
                break
            self._handleStateChange(stateChange)
        utils.setTimeout(0.1, self._simulate)

    def _changeAttr(self, location, attr_name, attr_val):
        atompm_id = self._m[location]['$key']
        return {'method': 'PUT',
                'uri': self._m[location]['$type'] + '/' + atompm_id + '.instance',
                'reqData': {'changes': {attr_name: attr_val}}}

    def _changeAttrWithID(self, atompm_id, attr_name, attr_val):
        return {'method': 'PUT',
                'uri': atompm_id + '.instance',
                'reqData': {'changes': {attr_name: attr_val}}}

    def _createNode(self, nodetype):
        return {'method': 'POST',
                'uri': nodetype + '.type',
                'reqData': {'hitchhiker': {}}}

    def _removeNode(self, uri):
        return {'method': 'DELETE',
                'uri': uri}

    def _requestNodeHighlight(self, location, color) :
        atompm_id = self._m[location]['$key']
        return utils.httpReq(
                'PUT',
                '127.0.0.1:8124',
                '/GET/console?wid=' + self._aswCommTools['wid'],
                {'text':'CLIENT_BDAPI :: '+
                    '{"func":"_highlight",'+
                    ' "args":'+
                        '{"asid":"' + atompm_id + '",'+
                        ' "color":"' + color + '"}}'})

    def _requestNodeHighlightWithID(self, atompm_id, color) :
        return utils.httpReq(
                'PUT',
                '127.0.0.1:8124',
                '/GET/console?wid=' + self._aswCommTools['wid'],
                {'text':'CLIENT_BDAPI :: '+
                    '{"func":"_highlight",'+
                    ' "args":'+
                        '{"asid":"' + atompm_id + '",'+
                        ' "color":"' + color + '"}}'})

    def _requestUnhighlight(self) :
        return utils.httpReq(
                'PUT',
                '127.0.0.1:8124',
                '/GET/console?wid=' + self._aswCommTools['wid'],
                {'text':'CLIENT_BDAPI :: '+
                    '{"func":"_highlight"}'})

    def _handleStateChange(self, stateChange):
        reqs = []
        logging.error(stateChange)
        parameters = stateChange.getParameters()
        event_name = stateChange.getName()
        if event_name != "inject_ok":
            for message_id in self.messages:
                msg = self._m_atompm['nodes'][message_id]
                if not 'injected' in msg or msg['injection_time'] < self._m_atompm['nodes'][self._m['$simulation']['$key']]['simulation_time']['value']:
                    reqs.append(self._removeNode('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + str(message_id) + '.instance'))
        if event_name in set(['all_states', 'new_states']):
            for message_id in self.messages:
                reqs.append(self._removeNode('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + str(message_id) + '.instance'))            
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
            for loc, statetuple in parameters[1].iteritems():
                if isinstance(statetuple, tuple):
                    new_tn = statetuple[0]
                    reqs.append(self._changeAttr(loc, 'time_next', str(new_tn[0])))
                    state = statetuple[1]
                else:
                    state = statetuple
                reqs.append(self._changeAttr(loc + '.' + state.name, 'current', True))
                attr_values = []
                for attr_name in filter(lambda x: not x.startswith('__') and x != 'name', dir(state)):
                    try:
                        attr_values.append({'name': attr_name, 'val': json.dumps(getattr(state, attr_name))})
                    except TypeError:
                        attr_values.append({'name': attr_name, 'val': str(getattr(state, attr_name))})
                reqs.append(self._changeAttr(loc + '.$state', 'attribute_values', attr_values))
                for locstate in self._m[loc + '.' + '$states']:
                    if locstate['name']['value'] != state.name:
                        reqs.append(self._changeAttr(loc + '.' + locstate['name']['value'], 'current', False))
        elif event_name == "imminents":
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
            for loc in parameters[1]:
                self._requestNodeHighlight(loc, "Blue")
        elif event_name in ("inbags", "outbags"):
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
            for loc, bags in parameters[1].iteritems():
                portinstance = self._m[loc]
                for m in bags:
                    atompm_id = '$%s$' % len(reqs)
                    reqs.append(self._createNode('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance'))
                    reqs.append(self._changeAttrWithID('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + atompm_id, 'position', portinstance['position']['value']))
                    reqs.append(self._changeAttrWithID('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + atompm_id, 'event_type', m.__class__.__name__))
                    if m.__class__.__name__ == 'str':
                        reqs.append(self._changeAttrWithID('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + atompm_id, 'attribute_values', [{'name': 'object', 'val': str(m)}]))
                    else:
                        reqs.append(self._changeAttrWithID('/Formalisms/ParallelDEVS/ParallelDEVS_Runtime/EventInstance/' + atompm_id, 'attribute_values', [{'name': attr_name, 'val': str(getattr(m, attr_name))} for attr_name in filter(lambda x: not x.startswith('__') and x != 'name', dir(m))]))
        elif event_name == "transitioning":
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
            for loc, transtype in parameters[1].iteritems():
                color = {"INTERNAL": "Blue", "EXTERNAL": "Red", "CONFLUENT": "Purple"}[transtype]
                self._requestNodeHighlight(loc, color)
        elif event_name == "new_tn":
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
            for loc, new_tn in parameters[1].iteritems():
                reqs.append(self._changeAttr(loc, 'time_next', str(new_tn[0])))
        elif event_name == "breakpoint_triggered":
            self._requestNodeHighlightWithID(parameters[0], "blue")
            if self._m_atompm['nodes'][parameters[0]]['disable_after_trigger']['value']:
                reqs.append(self._changeAttrWithID('/Formalisms/ParallelDEVS/ParallelDEVS_Debug/GlobalBreakpoint/' + parameters[0], 'enabled', False))
        elif event_name == "god_event_ok":
            print 'GOD EVENT RECEIVED'
        elif event_name == "inject_ok":
            print 'INJECT EVENT RECEIVED'
        elif event_name == 'terminate':
            curr_time = parameters[0][0]
            reqs.append(self._changeAttr('$simulation', 'simulation_time', curr_time))
        self._aswCommTools['httpReq']('POST', '/batchEdit', reqs)
        
