"""
Generated by Statechart compiler by Glenn De Jonghe and Joeri Exelmans

Date:   Tue Jan 13 11:53:26 2015

Model author: Yentl Van Tendeloo
Model name:   DEVS simulator
Model description:
A restricted PythonPDEVS simulator modelled in SCCD
"""

from python_runtime.statecharts_core import *
import cPickle as pickle
import time

# We basically just interface with the basesimulator
from scheduler import Scheduler
from DEVS import directConnect, CoupledDEVS, AtomicDEVS, RootDEVS

class Breakpoint(object):
    def __init__(self, breakpoint_id, function, enabled, disable_on_trigger):
        self.id = breakpoint_id
        self.function = function
        self.enabled = enabled
        self.disable_on_trigger = disable_on_trigger

# package "DEVS simulator"

class SCCDSimulator(RuntimeClassBase):
	def __init__(self, controller, model):
		RuntimeClassBase.__init__(self, controller)
		
		self.semantics.big_step_maximality = StatechartSemantics.TakeMany
		self.semantics.internal_event_lifeline = StatechartSemantics.Queue
		self.semantics.input_event_lifeline = StatechartSemantics.FirstComboStep
		self.semantics.priority = StatechartSemantics.SourceParent
		self.semantics.concurrency = StatechartSemantics.Single
		
		# Call user defined constructor
		SCCDSimulator.user_defined_constructor(self, model)
	
	def user_defined_constructor(self, model):
		# Simulation variables
		self.termination_time = None
		self.termination_condition = None
		self.simulation_time = (0.0, 0)
		self.model = model
		self.realtime_scale = 1.0
		
		# Values to be set during simulation
		self.realtime_starttime = None
		self.inject_queue = []
		
		# Model initialization
		self.model_ids = []
		self.model.finalize(name="", model_counter=0, model_ids=self.model_ids, locations={None: []}, selectHierarchy=[])
		
		# Direct connection
		if isinstance(self.model, CoupledDEVS):
		    self.model.componentSet = directConnect(self.model.componentSet, True)
		    self.model = RootDEVS(self.model.componentSet, self.model.componentSet, None)
		    for m in self.model.componentSet:
		        m.timeLast = (-m.elapsed, 1)
		        ta = m.timeAdvance()
		        m.timeNext = (m.timeLast[0] + ta, 1)
		        m.old_states = [(m.timeLast, pickle.dumps(m.state))]
		elif isinstance(self.model, AtomicDEVS):
		    for p in self.model.IPorts:
		        p.routingInLine = []
		        p.routingOutLine = []
		    for p in self.model.OPorts:
		        p.routingInLine = []
		        p.routingOutLine = []
		    self.model = RootDEVS([self.model], [self.model], None)
		    self.model.timeLast = (-self.model.elapsed, 1)
		    ta = self.model.timeAdvance()
		    self.model.timeNext = (self.model.timeLast[0] + ta, 1)
		
		# Fixed configuration options
		self.model.scheduler = Scheduler(self.model.componentSet, 1e-6, len(self.model.models))
		self.timeNext = self.model.scheduler.readFirst()
		
		# Cached values
		self.imminents = None
		self.outbags = None
		self.inbags = None
		self.transitioning = None
		self.new_states = None
		self.new_tn = None
		
		# Verbose trace file
		self.trace_file = None
		
		# Breakpoint management
		self.breakpoints = []
		
		# For a reset
		self.save_model = {model: (model.elapsed, pickle.dumps(model.state, pickle.HIGHEST_PROTOCOL)) for model in self.model.componentSet}
		self.transition_times = []
	
	def user_defined_destructor(self):
		pass
	
	
	# User defined method
	def serialize(self, type, object):
		if type == "imminents":
		    return [m.getModelFullName() for m in object]
		elif type == "inbags" or type == "outbags":
		    return {m.getPortFullName(): object[m] for m in object}
		elif type == "new_tn" or type == "new_states":
		    return {m.getModelFullName(): object[m] for m in object}
		elif type == "transitioning":
		    return {m.getModelFullName(): {1: "INTERNAL", 2: "EXTERNAL", 3: "CONFLUENT"}[object[m]] for m in object}
	
	
	# User defined method
	def find_port_with_name(self, name):
		for model in self.model.componentSet:
		    if name.startswith(model.getModelFullName()):
		        # Found a potential model
		        # We can't simply check for equality, as portnames might contain dots too
		        for port in model.IPorts:
		            if port.getPortFullName() == name:
		                # Now everything matches
		                return port
		# Nothing found
		return None
	
	
	# User defined method
	def find_model_with_name(self, name):
		for model in self.model.componentSet:
		    if name == model.getModelFullName():
		        # Found exact match
		        return model
		return None
	
	
	# User defined method
	def calculate_after(self):
		try:
		    # Process in parts of 100 milliseconds to repeatedly check the termination condition
		    nexttime = (self.timeNext[0] - (time.time() - self.realtime_starttime) / self.realtime_scale) * self.realtime_scale
		    x =  min(0.1, nexttime)
		    return x
		except TypeError, AttributeError:
		    # We are probably not simulating in realtime...
		    return float('inf')
	
	
	# User defined method
	def parse_options(self, configuration):
		self.termination_condition = None if "termination_condition" not in configuration else configuration["termination_condition"]
		self.termination_time = None if "termination_time" not in configuration else configuration["termination_time"]
		self.realtime_scale = 1.0 if "realtime_scale" not in configuration else 1.0/configuration["realtime_scale"]
		# Subtract the current simulation time to allow for pausing
		self.realtime_starttime = (time.time() - self.simulation_time[0]*self.realtime_scale)
		# Reset the time used in the waiting, as it might not get recomputed
		self.the_time = 0.00001
	
	
	# User defined method
	def should_terminate(self, realtime):
		# Now that it includes breakpoints, results are to be interpretted as follows:
		# -2 --> continue simulation
		# -1 --> termination condition
		# else --> breakpoint
		
		if realtime:
		    check_time = self.simulation_time
		else:
		    self.compute_timeNext()
		    check_time = self.timeNext
		
		# Just access the 'transitioned' dictionary
		# Kind of dirty though...
		if self.transitioning is None:
		    transitioned = set()
		else:
		    transitioned = set(self.transitioning.keys())
		
		if check_time[0] == float('inf'):
		    # Always terminate if we have reached infinity
		    terminate = True
		elif self.termination_condition is not None:
		    terminate = self.termination_condition(check_time, self.model, transitioned)
		else:
		    terminate = self.termination_time < check_time[0]
		
		if terminate:
		    # Always terminate, so don't check breakpoints
		    return -1
		else:
		    # Have to check breakpoints for termination
		    for bp in self.breakpoints:
		        if not bp.enabled:
		            continue
		        # Include the function in the scope
		        exec(bp.function)
		        # And execute it, note that the breakpoint thus has to start with "def breakpoint("
		        if breakpoint(check_time, self.model, transitioned):
		            # Triggered!
		            return bp.id
		        else:
		            # Not triggered, so continue
		            continue
		    return -2
	
	
	# User defined method
	def find_internal_imminents(self):
		self.imminents = self.model.scheduler.getImminent(self.simulation_time)
		self.transition_times.append(self.simulation_time)
	
	
	# User defined method
	def compute_outputfunction(self):
		self.outbags = {}
		for model in self.imminents:
		    self.outbags.update(model.outputFnc())
	
	
	# User defined method
	def route_messages(self):
		self.inbags = {}
		for outport in self.outbags:
		    payload = self.outbags[outport]
		    for inport, z in outport.routingOutLine:
		        if z is not None:
		            payload = [z(pickle.loads(pickle.dumps(m))) for m in payload]
		        self.inbags.setdefault(inport, []).extend(payload)
	
	
	# User defined method
	def process_injects(self):
		while self.inject_queue:
		    if self.inject_queue[0]["time"] > self.simulation_time:
		        break
		    config = self.inject_queue.pop(0)
		    portname = config["port"]
		    event = config["event"]
		    port = self.find_port_with_name(portname)
		    self.inbags.setdefault(port, []).append(event)
	
	
	# User defined method
	def find_all_imminents(self):
		# Internal codes:
		#   1 --> internal transition
		#   2 --> external transition
		#   3 --> confluent transition
		# These codes are a legacy of efficient PyPDEVS, but is kept here for consistency
		self.transitioning = {model: 1 for model in self.imminents}
		for inport in self.inbags:
		    aDEVS = inport.hostDEVS
		    aDEVS.myInput[inport] = self.inbags[inport]
		    if aDEVS in self.transitioning:
		        self.transitioning[aDEVS] = 3
		    else:
		        self.transitioning[aDEVS] = 2
	
	
	# User defined method
	def compute_transitions(self):
		self.new_states = {}
		for aDEVS in self.transitioning:
		    aDEVS.myInput = {key: pickle.loads(pickle.dumps(aDEVS.myInput[key], pickle.HIGHEST_PROTOCOL)) for key in aDEVS.myInput}
		    if self.transitioning[aDEVS] == 1:
		        aDEVS.state = aDEVS.intTransition()
		    elif self.transitioning[aDEVS] == 2:
		        aDEVS.elapsed = self.simulation_time[0] - aDEVS.timeLast[0]
		        aDEVS.state = aDEVS.extTransition(aDEVS.myInput)
		    elif self.transitioning[aDEVS] == 3:
		        aDEVS.elapsed = 0.
		        aDEVS.state = aDEVS.confTransition(aDEVS.myInput)
		    aDEVS.old_states.append((self.simulation_time, pickle.dumps(aDEVS.state)))
		    aDEVS.myInput = {}
		    self.new_states[aDEVS] = aDEVS.state
	
	
	# User defined method
	def compute_ta(self):
		self.new_tn = {}
		t, age = self.simulation_time
		for aDEVS in self.transitioning:
		    ta = aDEVS.timeAdvance()
		    aDEVS.timeLast = self.simulation_time
		    aDEVS.timeNext = (t + ta, 1 if ta else (age + 1))
		    self.new_tn[aDEVS] = aDEVS.timeNext
		    self.trace(self.transitioning[aDEVS], aDEVS)
		self.model.scheduler.massReschedule(self.transitioning)
		self.timeNext = self.model.scheduler.readFirst()
	
	
	# User defined method
	def trace(self, type, model):
		type_map = {1: "INTERNAL", 2: "EXTERNAL", 3: "CONFLUENT"}
		type = type_map[type]
		if self.trace_file is not None:
		    self.trace_file.write("%s TRANSITION in <%s> @ %s\n" % (type, model.getModelFullName(), model.timeLast[0]))
		    self.trace_file.write("  NEW STATE <%s>\n" % (model.state))
		    if type != "EXTERNAL":
		        self.trace_file.write("  OUTPUTFNC returned %s\n" % model.myOutput)
		    elif type != "INTERNAL":
		        self.trace_file.write("  inputs were %s\n" % model.myInput)
		    self.trace_file.write("  timeNext: %s (ta: %s)\n" % (model.timeNext[0], model.timeNext[0] - model.timeLast[0]))
	
	
	# User defined method
	def flush_file(self):
		if self.trace_file is not None:
		    self.trace_file.flush()
	
	
	# User defined method
	def process_breakpoints(self, realtime):
		breakpoint_id = self.should_terminate(realtime)
		for breakpoint in self.breakpoints:
		    if breakpoint.id == breakpoint_id:
		        if breakpoint.disable_on_trigger:
		            breakpoint.enabled = False
		        return breakpoint_id
	
	
	# User defined method
	def compute_timeNext(self):
		model_timeNext = self.model.scheduler.readFirst()
		if len(self.inject_queue) > 0:
		    self.timeNext = min(model_timeNext, self.inject_queue[0]["time"])
		else:
		    self.timeNext = model_timeNext
	
	
	# User defined method
	def rollback_step(self):
		if len(self.transition_times) == 0:
		    return
		new_time = self.transition_times.pop()
		for model in self.model.componentSet:
		    if model.old_states[-1][0] == new_time:
		        # Remove the current state
		        del model.old_states[-1]
		        # Set the new (old...) state
		        new_state = model.old_states[-1]
		        model.state = pickle.loads(new_state[1])
		        model.timeLast = new_state[0]
		        ta = model.timeAdvance()
		        model.timeNext = (model.timeLast[0] + ta, model.timeLast[1] + 1 if ta == 0 else 0)
		        self.model.scheduler.massReschedule([model])
		self.simulation_time = self.transition_times[-1] if len(self.transition_times) > 0 else (0.0, 0)
	
	def initializeStatechart(self):
		self.current_state[self.Root] = []
		self.current_state[self.Root_main] = []
		self.current_state[self.Root_main_injection_monitor] = []
		self.current_state[self.Root_main_tracer_monitor] = []
		self.current_state[self.Root_main_simulation_flow] = []
		self.current_state[self.Root_main_simulation_flow_check_termination] = []
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
		self.current_state[self.Root_main_simulation_state] = []
		self.current_state[self.Root_main_breakpoint_manager] = []
		self.current_state[self.Root_main_reset] = []
		
		# Enter default state
		self.enterDefault_Root_main()
	
	# Unique IDs for all statechart nodes
	Root = 0
	Root_main = 1
	Root_main_injection_monitor = 2
	Root_main_tracer_monitor = 3
	Root_main_simulation_flow = 4
	Root_main_simulation_flow_check_termination = 5
	Root_main_simulation_flow_do_simulation = 6
	Root_main_simulation_state = 7
	Root_main_breakpoint_manager = 8
	Root_main_reset = 9
	Root_main_injection_monitor_inject = 10
	Root_main_tracer_monitor_trace = 11
	Root_main_simulation_flow_initialize = 12
	Root_main_simulation_flow_check_termination_workaround = 13
	Root_main_simulation_flow_check_termination_wait = 14
	Root_main_simulation_flow_check_termination_small_step_check = 15
	Root_main_simulation_flow_check_termination_check_termination = 16
	Root_main_simulation_flow_do_simulation_init = 17
	Root_main_simulation_flow_do_simulation_found_internal_imminents = 18
	Root_main_simulation_flow_do_simulation_computed_outputfunction = 19
	Root_main_simulation_flow_do_simulation_routed_messages = 20
	Root_main_simulation_flow_do_simulation_found_all_imminents = 21
	Root_main_simulation_flow_do_simulation_computed_transitions = 22
	Root_main_simulation_state_paused = 23
	Root_main_simulation_state_continuous = 24
	Root_main_simulation_state_realtime = 25
	Root_main_simulation_state_big_step = 26
	Root_main_breakpoint_manager_breakpoint_manage = 27
	Root_main_reset_reset = 28
	
	# Statechart enter/exit action method(s)
	
	def enter_Root_main(self):
		self.current_state[self.Root].append(self.Root_main)
	
	def exit_Root_main(self):
		self.exit_Root_main_injection_monitor()
		self.exit_Root_main_tracer_monitor()
		self.exit_Root_main_simulation_flow()
		self.exit_Root_main_simulation_state()
		self.exit_Root_main_breakpoint_manager()
		self.exit_Root_main_reset()
		self.current_state[self.Root] = []
	
	def enter_Root_main_injection_monitor(self):
		self.current_state[self.Root_main].append(self.Root_main_injection_monitor)
	
	def exit_Root_main_injection_monitor(self):
		if self.Root_main_injection_monitor_inject in self.current_state[self.Root_main_injection_monitor]:
			self.exit_Root_main_injection_monitor_inject()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_tracer_monitor(self):
		self.current_state[self.Root_main].append(self.Root_main_tracer_monitor)
	
	def exit_Root_main_tracer_monitor(self):
		if self.Root_main_tracer_monitor_trace in self.current_state[self.Root_main_tracer_monitor]:
			self.exit_Root_main_tracer_monitor_trace()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_simulation_flow(self):
		self.current_state[self.Root_main].append(self.Root_main_simulation_flow)
	
	def exit_Root_main_simulation_flow(self):
		if self.Root_main_simulation_flow_initialize in self.current_state[self.Root_main_simulation_flow]:
			self.exit_Root_main_simulation_flow_initialize()
		if self.Root_main_simulation_flow_check_termination in self.current_state[self.Root_main_simulation_flow]:
			self.exit_Root_main_simulation_flow_check_termination()
		if self.Root_main_simulation_flow_do_simulation in self.current_state[self.Root_main_simulation_flow]:
			self.exit_Root_main_simulation_flow_do_simulation()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_simulation_flow_check_termination(self):
		self.current_state[self.Root_main_simulation_flow].append(self.Root_main_simulation_flow_check_termination)
	
	def exit_Root_main_simulation_flow_check_termination(self):
		if self.Root_main_simulation_flow_check_termination_workaround in self.current_state[self.Root_main_simulation_flow_check_termination]:
			self.exit_Root_main_simulation_flow_check_termination_workaround()
		if self.Root_main_simulation_flow_check_termination_wait in self.current_state[self.Root_main_simulation_flow_check_termination]:
			self.exit_Root_main_simulation_flow_check_termination_wait()
		if self.Root_main_simulation_flow_check_termination_small_step_check in self.current_state[self.Root_main_simulation_flow_check_termination]:
			self.exit_Root_main_simulation_flow_check_termination_small_step_check()
		if self.Root_main_simulation_flow_check_termination_check_termination in self.current_state[self.Root_main_simulation_flow_check_termination]:
			self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.simulation_time = self.timeNext
		self.current_state[self.Root_main_simulation_flow] = []
	
	def enter_Root_main_simulation_flow_do_simulation(self):
		self.current_state[self.Root_main_simulation_flow].append(self.Root_main_simulation_flow_do_simulation)
	
	def exit_Root_main_simulation_flow_do_simulation(self):
		if self.Root_main_simulation_flow_do_simulation_init in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_init()
		if self.Root_main_simulation_flow_do_simulation_found_internal_imminents in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_found_internal_imminents()
		if self.Root_main_simulation_flow_do_simulation_computed_outputfunction in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_computed_outputfunction()
		if self.Root_main_simulation_flow_do_simulation_routed_messages in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_routed_messages()
		if self.Root_main_simulation_flow_do_simulation_found_all_imminents in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_found_all_imminents()
		if self.Root_main_simulation_flow_do_simulation_computed_transitions in self.current_state[self.Root_main_simulation_flow_do_simulation]:
			self.exit_Root_main_simulation_flow_do_simulation_computed_transitions()
		self.current_state[self.Root_main_simulation_flow] = []
	
	def enter_Root_main_simulation_state(self):
		self.current_state[self.Root_main].append(self.Root_main_simulation_state)
	
	def exit_Root_main_simulation_state(self):
		if self.Root_main_simulation_state_paused in self.current_state[self.Root_main_simulation_state]:
			self.exit_Root_main_simulation_state_paused()
		if self.Root_main_simulation_state_continuous in self.current_state[self.Root_main_simulation_state]:
			self.exit_Root_main_simulation_state_continuous()
		if self.Root_main_simulation_state_realtime in self.current_state[self.Root_main_simulation_state]:
			self.exit_Root_main_simulation_state_realtime()
		if self.Root_main_simulation_state_big_step in self.current_state[self.Root_main_simulation_state]:
			self.exit_Root_main_simulation_state_big_step()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_breakpoint_manager(self):
		self.current_state[self.Root_main].append(self.Root_main_breakpoint_manager)
	
	def exit_Root_main_breakpoint_manager(self):
		if self.Root_main_breakpoint_manager_breakpoint_manage in self.current_state[self.Root_main_breakpoint_manager]:
			self.exit_Root_main_breakpoint_manager_breakpoint_manage()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_reset(self):
		self.current_state[self.Root_main].append(self.Root_main_reset)
	
	def exit_Root_main_reset(self):
		if self.Root_main_reset_reset in self.current_state[self.Root_main_reset]:
			self.exit_Root_main_reset_reset()
		self.current_state[self.Root_main] = []
	
	def enter_Root_main_injection_monitor_inject(self):
		self.current_state[self.Root_main_injection_monitor].append(self.Root_main_injection_monitor_inject)
	
	def exit_Root_main_injection_monitor_inject(self):
		self.current_state[self.Root_main_injection_monitor] = []
	
	def enter_Root_main_tracer_monitor_trace(self):
		self.current_state[self.Root_main_tracer_monitor].append(self.Root_main_tracer_monitor_trace)
	
	def exit_Root_main_tracer_monitor_trace(self):
		self.current_state[self.Root_main_tracer_monitor] = []
	
	def enter_Root_main_simulation_flow_initialize(self):
		self.current_state[self.Root_main_simulation_flow].append(self.Root_main_simulation_flow_initialize)
	
	def exit_Root_main_simulation_flow_initialize(self):
		self.current_state[self.Root_main_simulation_flow] = []
	
	def enter_Root_main_simulation_flow_check_termination_workaround(self):
		self.timers[0] = 0
		self.current_state[self.Root_main_simulation_flow_check_termination].append(self.Root_main_simulation_flow_check_termination_workaround)
	
	def exit_Root_main_simulation_flow_check_termination_workaround(self):
		self.timers.pop(0, None)
		self.current_state[self.Root_main_simulation_flow_check_termination] = []
	
	def enter_Root_main_simulation_flow_check_termination_wait(self):
		self.timers[1] = self.calculate_after()
		self.current_state[self.Root_main_simulation_flow_check_termination].append(self.Root_main_simulation_flow_check_termination_wait)
	
	def exit_Root_main_simulation_flow_check_termination_wait(self):
		self.timers.pop(1, None)
		diff = time.time() - self.realtime_starttime
		self.simulation_time = (diff / self.realtime_scale, 1)
		self.current_state[self.Root_main_simulation_flow_check_termination] = []
	
	def enter_Root_main_simulation_flow_check_termination_small_step_check(self):
		self.current_state[self.Root_main_simulation_flow_check_termination].append(self.Root_main_simulation_flow_check_termination_small_step_check)
	
	def exit_Root_main_simulation_flow_check_termination_small_step_check(self):
		self.current_state[self.Root_main_simulation_flow_check_termination] = []
	
	def enter_Root_main_simulation_flow_check_termination_check_termination(self):
		self.compute_timeNext()
		self.the_time = self.calculate_after()
		self.current_state[self.Root_main_simulation_flow_check_termination].append(self.Root_main_simulation_flow_check_termination_check_termination)
	
	def exit_Root_main_simulation_flow_check_termination_check_termination(self):
		self.current_state[self.Root_main_simulation_flow_check_termination] = []
	
	def enter_Root_main_simulation_flow_do_simulation_init(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_init)
	
	def exit_Root_main_simulation_flow_do_simulation_init(self):
		self.find_internal_imminents()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_flow_do_simulation_found_internal_imminents(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_found_internal_imminents)
	
	def exit_Root_main_simulation_flow_do_simulation_found_internal_imminents(self):
		self.compute_outputfunction()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_flow_do_simulation_computed_outputfunction(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_computed_outputfunction)
	
	def exit_Root_main_simulation_flow_do_simulation_computed_outputfunction(self):
		self.route_messages()
		self.process_injects()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_flow_do_simulation_routed_messages(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_routed_messages)
	
	def exit_Root_main_simulation_flow_do_simulation_routed_messages(self):
		self.find_all_imminents()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_flow_do_simulation_found_all_imminents(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_found_all_imminents)
	
	def exit_Root_main_simulation_flow_do_simulation_found_all_imminents(self):
		self.compute_transitions()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_flow_do_simulation_computed_transitions(self):
		self.current_state[self.Root_main_simulation_flow_do_simulation].append(self.Root_main_simulation_flow_do_simulation_computed_transitions)
	
	def exit_Root_main_simulation_flow_do_simulation_computed_transitions(self):
		self.compute_ta()
		self.current_state[self.Root_main_simulation_flow_do_simulation] = []
	
	def enter_Root_main_simulation_state_paused(self):
		self.current_state[self.Root_main_simulation_state].append(self.Root_main_simulation_state_paused)
	
	def exit_Root_main_simulation_state_paused(self):
		self.current_state[self.Root_main_simulation_state] = []
	
	def enter_Root_main_simulation_state_continuous(self):
		self.current_state[self.Root_main_simulation_state].append(self.Root_main_simulation_state_continuous)
	
	def exit_Root_main_simulation_state_continuous(self):
		self.current_state[self.Root_main_simulation_state] = []
	
	def enter_Root_main_simulation_state_realtime(self):
		self.current_state[self.Root_main_simulation_state].append(self.Root_main_simulation_state_realtime)
	
	def exit_Root_main_simulation_state_realtime(self):
		self.current_state[self.Root_main_simulation_state] = []
	
	def enter_Root_main_simulation_state_big_step(self):
		self.current_state[self.Root_main_simulation_state].append(self.Root_main_simulation_state_big_step)
	
	def exit_Root_main_simulation_state_big_step(self):
		self.current_state[self.Root_main_simulation_state] = []
	
	def enter_Root_main_breakpoint_manager_breakpoint_manage(self):
		self.current_state[self.Root_main_breakpoint_manager].append(self.Root_main_breakpoint_manager_breakpoint_manage)
	
	def exit_Root_main_breakpoint_manager_breakpoint_manage(self):
		self.current_state[self.Root_main_breakpoint_manager] = []
	
	def enter_Root_main_reset_reset(self):
		self.current_state[self.Root_main_reset].append(self.Root_main_reset_reset)
	
	def exit_Root_main_reset_reset(self):
		self.current_state[self.Root_main_reset] = []
	
	# Statechart enter/exit default method(s)
	
	def enterDefault_Root_main(self):
		self.enter_Root_main()
		self.enterDefault_Root_main_injection_monitor()
		self.enterDefault_Root_main_tracer_monitor()
		self.enterDefault_Root_main_simulation_flow()
		self.enterDefault_Root_main_simulation_state()
		self.enterDefault_Root_main_breakpoint_manager()
		self.enterDefault_Root_main_reset()
	
	def enterDefault_Root_main_injection_monitor(self):
		self.enter_Root_main_injection_monitor()
		self.enter_Root_main_injection_monitor_inject()
	
	def enterDefault_Root_main_tracer_monitor(self):
		self.enter_Root_main_tracer_monitor()
		self.enter_Root_main_tracer_monitor_trace()
	
	def enterDefault_Root_main_simulation_flow(self):
		self.enter_Root_main_simulation_flow()
		self.enter_Root_main_simulation_flow_initialize()
	
	def enterDefault_Root_main_simulation_flow_check_termination(self):
		self.enter_Root_main_simulation_flow_check_termination()
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def enterDefault_Root_main_simulation_flow_do_simulation(self):
		self.enter_Root_main_simulation_flow_do_simulation()
		self.enter_Root_main_simulation_flow_do_simulation_init()
	
	def enterDefault_Root_main_simulation_state(self):
		self.enter_Root_main_simulation_state()
		self.enter_Root_main_simulation_state_paused()
	
	def enterDefault_Root_main_breakpoint_manager(self):
		self.enter_Root_main_breakpoint_manager()
		self.enter_Root_main_breakpoint_manager_breakpoint_manage()
	
	def enterDefault_Root_main_reset(self):
		self.enter_Root_main_reset()
		self.enter_Root_main_reset_reset()
	
	# Statechart transitions
	
	def generateCandidatesChildren_Root(self):
		if self.current_state[self.Root][0] == self.Root_main:
			return self.generateCandidates_Root_main()
		return False
	
	def generateCandidates_Root(self):
		if not self.combo_step.isArenaChanged(self.Root):
			return self.generateCandidatesChildren_Root()
		else:
			return True
	
	def generateCandidatesChildren_Root_main(self):
		branch_done = False
		branch_done = (self.generateCandidates_Root_main_injection_monitor() or branch_done)
		branch_done = (self.generateCandidates_Root_main_tracer_monitor() or branch_done)
		branch_done = (self.generateCandidates_Root_main_simulation_flow() or branch_done)
		branch_done = (self.generateCandidates_Root_main_simulation_state() or branch_done)
		branch_done = (self.generateCandidates_Root_main_breakpoint_manager() or branch_done)
		branch_done = (self.generateCandidates_Root_main_reset() or branch_done)
		return branch_done
	
	def generateCandidates_Root_main(self):
		if not self.combo_step.isArenaChanged(self.Root_main):
			return self.generateCandidatesChildren_Root_main()
		else:
			return True
	
	def generateCandidatesChildren_Root_main_injection_monitor(self):
		if self.current_state[self.Root_main_injection_monitor][0] == self.Root_main_injection_monitor_inject:
			return self.generateCandidates_Root_main_injection_monitor_inject()
		return False
	
	def generateCandidates_Root_main_injection_monitor(self):
		if not self.combo_step.isArenaChanged(self.Root_main_injection_monitor):
			return self.generateCandidatesChildren_Root_main_injection_monitor()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_injection_monitor_inject(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "inject") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_injection_monitor_inject_1, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_injection_monitor_inject(self):
		if not self.combo_step.isArenaChanged(self.Root_main_injection_monitor_inject):
			return self.generateCandidatesCurrent_Root_main_injection_monitor_inject()
		else:
			return True
	
	def transition_Root_main_injection_monitor_inject_1(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_injection_monitor_inject()
		configuration["time"] = (configuration["time"], 1)
		self.inject_queue.append(configuration)
		self.inject_queue.sort(key=lambda i: i["time"])
		self.big_step.outputEvent(Event("inject_ok", "reply", []))
		self.combo_step.setArenaChanged(self.Root_main_injection_monitor)
		self.enter_Root_main_injection_monitor_inject()
	
	def generateCandidatesChildren_Root_main_tracer_monitor(self):
		if self.current_state[self.Root_main_tracer_monitor][0] == self.Root_main_tracer_monitor_trace:
			return self.generateCandidates_Root_main_tracer_monitor_trace()
		return False
	
	def generateCandidates_Root_main_tracer_monitor(self):
		if not self.combo_step.isArenaChanged(self.Root_main_tracer_monitor):
			return self.generateCandidatesChildren_Root_main_tracer_monitor()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_tracer_monitor_trace(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "trace") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_tracer_monitor_trace_1, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_tracer_monitor_trace(self):
		if not self.combo_step.isArenaChanged(self.Root_main_tracer_monitor_trace):
			return self.generateCandidatesCurrent_Root_main_tracer_monitor_trace()
		else:
			return True
	
	def transition_Root_main_tracer_monitor_trace_1(self, parameters):
		filename = parameters[0]
		self.exit_Root_main_tracer_monitor_trace()
		if filename is not None:
		    self.trace_file = open(filename, 'w')
		else:
		    self.trace_file = None
		self.big_step.outputEvent(Event("trace_config_ok", "reply", []))
		self.combo_step.setArenaChanged(self.Root_main_tracer_monitor)
		self.enter_Root_main_tracer_monitor_trace()
	
	def generateCandidatesChildren_Root_main_simulation_flow(self):
		if self.current_state[self.Root_main_simulation_flow][0] == self.Root_main_simulation_flow_initialize:
			return self.generateCandidates_Root_main_simulation_flow_initialize()
		elif self.current_state[self.Root_main_simulation_flow][0] == self.Root_main_simulation_flow_check_termination:
			return self.generateCandidates_Root_main_simulation_flow_check_termination()
		elif self.current_state[self.Root_main_simulation_flow][0] == self.Root_main_simulation_flow_do_simulation:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation()
		return False
	
	def generateCandidates_Root_main_simulation_flow(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow):
			return self.generateCandidatesChildren_Root_main_simulation_flow()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_simulation_flow_initialize(self):
		enabled_events = self.getEnabledEvents()
		self.small_step.addCandidate(self.transition_Root_main_simulation_flow_initialize_1, [])
		return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_initialize(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_initialize):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_initialize()
		else:
			return True
	
	def transition_Root_main_simulation_flow_initialize_1(self, parameters):
		self.exit_Root_main_simulation_flow_initialize()
		self.big_step.outputEvent(Event("all_states", "reply", [self.simulation_time, {m.getModelFullName(): (m.timeNext, m.state) for m in self.model.componentSet}]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_check_termination()
	
	def generateCandidatesChildren_Root_main_simulation_flow_check_termination(self):
		if self.current_state[self.Root_main_simulation_flow_check_termination][0] == self.Root_main_simulation_flow_check_termination_workaround:
			return self.generateCandidates_Root_main_simulation_flow_check_termination_workaround()
		elif self.current_state[self.Root_main_simulation_flow_check_termination][0] == self.Root_main_simulation_flow_check_termination_wait:
			return self.generateCandidates_Root_main_simulation_flow_check_termination_wait()
		elif self.current_state[self.Root_main_simulation_flow_check_termination][0] == self.Root_main_simulation_flow_check_termination_small_step_check:
			return self.generateCandidates_Root_main_simulation_flow_check_termination_small_step_check()
		elif self.current_state[self.Root_main_simulation_flow_check_termination][0] == self.Root_main_simulation_flow_check_termination_check_termination:
			return self.generateCandidates_Root_main_simulation_flow_check_termination_check_termination()
		return False
	
	def generateCandidates_Root_main_simulation_flow_check_termination(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_check_termination):
			return self.generateCandidatesChildren_Root_main_simulation_flow_check_termination()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_simulation_flow_check_termination_workaround(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "_0after":
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_workaround_1, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_check_termination_workaround(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_check_termination_workaround):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_check_termination_workaround()
		else:
			return True
	
	def transition_Root_main_simulation_flow_check_termination_workaround_1(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_workaround()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_check_termination()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_check_termination_wait(self):
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_wait_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "_1after":
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_wait_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_check_termination_wait(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_check_termination_wait):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_check_termination_wait()
		else:
			return True
	
	def transition_Root_main_simulation_flow_check_termination_wait_1(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_wait()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_check_termination()
	
	def transition_Root_main_simulation_flow_check_termination_wait_2(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_wait()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_check_termination()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_check_termination_small_step_check(self):
		enabled_events = self.getEnabledEvents()
		if self.should_terminate(False) == -2:
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_small_step_check_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.should_terminate(False) == -1:
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_small_step_check_2, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.should_terminate(False) > -1:
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_small_step_check_3, [])
			return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_check_termination_small_step_check(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_check_termination_small_step_check):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_check_termination_small_step_check()
		else:
			return True
	
	def transition_Root_main_simulation_flow_check_termination_small_step_check_1(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_do_simulation()
	
	def transition_Root_main_simulation_flow_check_termination_small_step_check_2(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_small_step_check()
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_check_termination()
	
	def transition_Root_main_simulation_flow_check_termination_small_step_check_3(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_small_step_check()
		breakpoint_id = self.process_breakpoints()
		self.big_step.outputEvent(Event("breakpoint_triggered", "reply", [breakpoint_id]))
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_check_termination()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_check_termination_check_termination(self):
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_continuous]) and (self.should_terminate(False) == -2):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_big_step]) and (self.should_terminate(False) == -2):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_2, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(True) == -2) and (self.the_time > 0.0):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_3, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(True) == -2) and (self.the_time <= 0.0):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_4, [])
			return True
		enabled_events = self.getEnabledEvents()
		if (not self.inState([self.Root_main_simulation_state_paused]) and self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(True) == -1)):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_5, [])
			return True
		enabled_events = self.getEnabledEvents()
		if (not self.inState([self.Root_main_simulation_state_paused]) and not self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(False) == -1)):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_6, [])
			return True
		enabled_events = self.getEnabledEvents()
		if (not self.inState([self.Root_main_simulation_state_paused])) and self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(True) > -1):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_7, [])
			return True
		enabled_events = self.getEnabledEvents()
		if (not self.inState([self.Root_main_simulation_state_paused])) and not self.inState([self.Root_main_simulation_state_realtime]) and (self.should_terminate(False) > -1):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_8, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_9, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "god_event") and (e.port == "request"):
				parameters = e.parameters
				configuration = parameters[0]
				if self.inState([self.Root_main_simulation_state_paused]):
					self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_10, e.parameters)
					return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "backwards_step") and (e.port == "request"):
				parameters = e.parameters
				if self.inState([self.Root_main_simulation_state_paused]):
					self.small_step.addCandidate(self.transition_Root_main_simulation_flow_check_termination_check_termination_11, e.parameters)
					return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_check_termination_check_termination(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_check_termination_check_termination):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_check_termination_check_termination()
		else:
			return True
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_1(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_do_simulation()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_2(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_do_simulation()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_3(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_wait()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_4(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_do_simulation()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_5(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_6(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_7(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		breakpoint_id = self.process_breakpoints(True)
		self.big_step.outputEvent(Event("breakpoint_triggered", "reply", [breakpoint_id]))
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_8(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		breakpoint_id = self.process_breakpoints(False)
		self.big_step.outputEvent(Event("breakpoint_triggered", "reply", [breakpoint_id]))
		self.raiseInternalEvent(Event("termination_condition", None, []))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_9(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.parse_options(configuration)
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_small_step_check()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_10(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		modelname = configuration["model"]
		state_attribute = configuration["attribute"]
		new_value = configuration["value"]
		model = self.find_model_with_name(modelname)
		setattr(model.state, state_attribute, new_value)
		# Call the timeadvance method again and compute new ta
		ta = model.timeAdvance()
		model.timeNext = (model.timeLast[0] + ta, 1 if ta else (model.timeLast[1] + 1))
		self.model.scheduler.massReschedule([model])
		# self.simulation_time = self.model.scheduler.readFirst()
		self.big_step.outputEvent(Event("god_event_ok", "reply", [{model.getModelFullName(): str(model.state)}]))
		self.big_step.outputEvent(Event("new_tn", "reply", [self.simulation_time, {model.getModelFullName(): model.timeNext}]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def transition_Root_main_simulation_flow_check_termination_check_termination_11(self, parameters):
		self.exit_Root_main_simulation_flow_check_termination_check_termination()
		self.rollback_step()
		self.big_step.outputEvent(Event("all_states", "reply", [self.simulation_time, {m.getModelFullName(): (m.timeNext, m.state) for m in self.model.componentSet}]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_check_termination)
		self.enter_Root_main_simulation_flow_check_termination_workaround()
	
	def generateCandidatesChildren_Root_main_simulation_flow_do_simulation(self):
		if self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_init:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_init()
		elif self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_found_internal_imminents:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_found_internal_imminents()
		elif self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_computed_outputfunction:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_computed_outputfunction()
		elif self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_routed_messages:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_routed_messages()
		elif self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_found_all_imminents:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_found_all_imminents()
		elif self.current_state[self.Root_main_simulation_flow_do_simulation][0] == self.Root_main_simulation_flow_do_simulation_computed_transitions:
			return self.generateCandidates_Root_main_simulation_flow_do_simulation_computed_transitions()
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation):
			return self.generateCandidatesChildren_Root_main_simulation_flow_do_simulation()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_init(self):
		enabled_events = self.getEnabledEvents()
		if not self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_init_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_init_2, [])
			return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_init(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_init):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_init()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_init_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_init()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_found_internal_imminents()
	
	def transition_Root_main_simulation_flow_do_simulation_init_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_init()
		self.big_step.outputEvent(Event("imminents", "reply", [self.simulation_time, self.serialize('imminents', self.imminents)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_found_internal_imminents()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_found_internal_imminents(self):
		enabled_events = self.getEnabledEvents()
		if not self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_found_internal_imminents_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_found_internal_imminents_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_found_internal_imminents(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_found_internal_imminents):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_found_internal_imminents()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_found_internal_imminents_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_found_internal_imminents()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_computed_outputfunction()
	
	def transition_Root_main_simulation_flow_do_simulation_found_internal_imminents_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_found_internal_imminents()
		self.big_step.outputEvent(Event("outbags", "reply", [self.simulation_time, self.serialize('outbags', self.outbags)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_computed_outputfunction()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_computed_outputfunction(self):
		enabled_events = self.getEnabledEvents()
		if not self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_computed_outputfunction_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_computed_outputfunction_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_computed_outputfunction(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_computed_outputfunction):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_computed_outputfunction()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_computed_outputfunction_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_computed_outputfunction()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_routed_messages()
	
	def transition_Root_main_simulation_flow_do_simulation_computed_outputfunction_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_computed_outputfunction()
		self.big_step.outputEvent(Event("inbags", "reply", [self.simulation_time, self.serialize('inbags', self.inbags)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_routed_messages()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_routed_messages(self):
		enabled_events = self.getEnabledEvents()
		if not self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_routed_messages_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_routed_messages_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_routed_messages(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_routed_messages):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_routed_messages()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_routed_messages_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_routed_messages()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_found_all_imminents()
	
	def transition_Root_main_simulation_flow_do_simulation_routed_messages_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_routed_messages()
		self.big_step.outputEvent(Event("transitioning", "reply", [self.simulation_time, self.serialize('transitioning', self.transitioning)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_found_all_imminents()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_found_all_imminents(self):
		enabled_events = self.getEnabledEvents()
		if not self.inState([self.Root_main_simulation_state_paused]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_found_all_imminents_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_found_all_imminents_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_found_all_imminents(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_found_all_imminents):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_found_all_imminents()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_found_all_imminents_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_found_all_imminents()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_computed_transitions()
	
	def transition_Root_main_simulation_flow_do_simulation_found_all_imminents_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation_found_all_imminents()
		self.big_step.outputEvent(Event("new_states", "reply", [self.simulation_time, self.serialize('new_states', self.new_states)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow_do_simulation)
		self.enter_Root_main_simulation_flow_do_simulation_computed_transitions()
	
	def generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_computed_transitions(self):
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_continuous]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_computed_transitions_1, [])
			return True
		enabled_events = self.getEnabledEvents()
		if self.inState([self.Root_main_simulation_state_realtime]) or self.inState([self.Root_main_simulation_state_big_step]):
			self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_computed_transitions_2, [])
			return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "small_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_flow_do_simulation_computed_transitions_3, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_flow_do_simulation_computed_transitions(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_flow_do_simulation_computed_transitions):
			return self.generateCandidatesCurrent_Root_main_simulation_flow_do_simulation_computed_transitions()
		else:
			return True
	
	def transition_Root_main_simulation_flow_do_simulation_computed_transitions_1(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation()
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_check_termination()
	
	def transition_Root_main_simulation_flow_do_simulation_computed_transitions_2(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation()
		self.raiseInternalEvent(Event("big_step_done", None, []))
		self.big_step.outputEvent(Event("new_states", "reply", [self.simulation_time, self.serialize('new_states', self.new_states)]))
		self.big_step.outputEvent(Event("new_tn", "reply", [self.simulation_time, self.serialize('new_tn', self.new_tn)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_check_termination()
	
	def transition_Root_main_simulation_flow_do_simulation_computed_transitions_3(self, parameters):
		self.exit_Root_main_simulation_flow_do_simulation()
		self.big_step.outputEvent(Event("new_tn", "reply", [self.simulation_time, self.serialize('new_tn', self.new_tn)]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_flow)
		self.enterDefault_Root_main_simulation_flow_check_termination()
	
	def generateCandidatesChildren_Root_main_simulation_state(self):
		if self.current_state[self.Root_main_simulation_state][0] == self.Root_main_simulation_state_paused:
			return self.generateCandidates_Root_main_simulation_state_paused()
		elif self.current_state[self.Root_main_simulation_state][0] == self.Root_main_simulation_state_continuous:
			return self.generateCandidates_Root_main_simulation_state_continuous()
		elif self.current_state[self.Root_main_simulation_state][0] == self.Root_main_simulation_state_realtime:
			return self.generateCandidates_Root_main_simulation_state_realtime()
		elif self.current_state[self.Root_main_simulation_state][0] == self.Root_main_simulation_state_big_step:
			return self.generateCandidates_Root_main_simulation_state_big_step()
		return False
	
	def generateCandidates_Root_main_simulation_state(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_state):
			return self.generateCandidatesChildren_Root_main_simulation_state()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_simulation_state_paused(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "simulate") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_paused_1, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "realtime") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_paused_2, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "big_step") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_paused_3, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_state_paused(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_state_paused):
			return self.generateCandidatesCurrent_Root_main_simulation_state_paused()
		else:
			return True
	
	def transition_Root_main_simulation_state_paused_1(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_simulation_state_paused()
		self.parse_options(configuration)
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_continuous()
	
	def transition_Root_main_simulation_state_paused_2(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_simulation_state_paused()
		self.parse_options(configuration)
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_realtime()
	
	def transition_Root_main_simulation_state_paused_3(self, parameters):
		configuration = parameters[0]
		self.exit_Root_main_simulation_state_paused()
		self.parse_options(configuration)
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_big_step()
	
	def generateCandidatesCurrent_Root_main_simulation_state_continuous(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "pause") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_continuous_1, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "termination_condition":
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_continuous_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_state_continuous(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_state_continuous):
			return self.generateCandidatesCurrent_Root_main_simulation_state_continuous()
		else:
			return True
	
	def transition_Root_main_simulation_state_continuous_1(self, parameters):
		self.exit_Root_main_simulation_state_continuous()
		# Just override termination condition
		self.termination_condition = lambda i, j, k : True
		self.termination_time = None
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_continuous()
	
	def transition_Root_main_simulation_state_continuous_2(self, parameters):
		self.exit_Root_main_simulation_state_continuous()
		self.big_step.outputEvent(Event("terminate", "reply", [self.simulation_time]))
		self.flush_file()
		self.big_step.outputEvent(Event("all_states", "reply", [self.simulation_time, {m.getModelFullName(): (m.timeNext, m.state) for m in self.model.componentSet}]))
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_paused()
	
	def generateCandidatesCurrent_Root_main_simulation_state_realtime(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "pause") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_realtime_1, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "termination_condition":
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_realtime_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_state_realtime(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_state_realtime):
			return self.generateCandidatesCurrent_Root_main_simulation_state_realtime()
		else:
			return True
	
	def transition_Root_main_simulation_state_realtime_1(self, parameters):
		self.exit_Root_main_simulation_state_realtime()
		# Just override termination condition
		self.termination_condition = lambda i, j, k : True
		self.termination_time = None
		
		# Don't forget to correctly set the simulation time
		diff = time.time() - self.realtime_starttime
		self.simulation_time = (diff / self.realtime_scale, 1)
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_realtime()
	
	def transition_Root_main_simulation_state_realtime_2(self, parameters):
		self.exit_Root_main_simulation_state_realtime()
		self.big_step.outputEvent(Event("terminate", "reply", [self.simulation_time]))
		self.flush_file()
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_paused()
	
	def generateCandidatesCurrent_Root_main_simulation_state_big_step(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "big_step_done":
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_big_step_1, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if e.name == "termination_condition":
				self.small_step.addCandidate(self.transition_Root_main_simulation_state_big_step_2, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_simulation_state_big_step(self):
		if not self.combo_step.isArenaChanged(self.Root_main_simulation_state_big_step):
			return self.generateCandidatesCurrent_Root_main_simulation_state_big_step()
		else:
			return True
	
	def transition_Root_main_simulation_state_big_step_1(self, parameters):
		self.exit_Root_main_simulation_state_big_step()
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_paused()
	
	def transition_Root_main_simulation_state_big_step_2(self, parameters):
		self.exit_Root_main_simulation_state_big_step()
		self.big_step.outputEvent(Event("terminate", "reply", [self.simulation_time]))
		self.flush_file()
		self.combo_step.setArenaChanged(self.Root_main_simulation_state)
		self.enter_Root_main_simulation_state_paused()
	
	def generateCandidatesChildren_Root_main_breakpoint_manager(self):
		if self.current_state[self.Root_main_breakpoint_manager][0] == self.Root_main_breakpoint_manager_breakpoint_manage:
			return self.generateCandidates_Root_main_breakpoint_manager_breakpoint_manage()
		return False
	
	def generateCandidates_Root_main_breakpoint_manager(self):
		if not self.combo_step.isArenaChanged(self.Root_main_breakpoint_manager):
			return self.generateCandidatesChildren_Root_main_breakpoint_manager()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_breakpoint_manager_breakpoint_manage(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "add_breakpoint") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_breakpoint_manager_breakpoint_manage_1, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "del_breakpoint") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_breakpoint_manager_breakpoint_manage_2, e.parameters)
				return True
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "toggle_breakpoint") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_breakpoint_manager_breakpoint_manage_3, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_breakpoint_manager_breakpoint_manage(self):
		if not self.combo_step.isArenaChanged(self.Root_main_breakpoint_manager_breakpoint_manage):
			return self.generateCandidatesCurrent_Root_main_breakpoint_manager_breakpoint_manage()
		else:
			return True
	
	def transition_Root_main_breakpoint_manager_breakpoint_manage_1(self, parameters):
		breakpoint_id = parameters[0]
		function = parameters[1]
		enabled = parameters[2]
		disable_on_trigger = parameters[3]
		self.exit_Root_main_breakpoint_manager_breakpoint_manage()
		self.breakpoints.append(Breakpoint(breakpoint_id, function, enabled, disable_on_trigger))
		self.combo_step.setArenaChanged(self.Root_main_breakpoint_manager)
		self.enter_Root_main_breakpoint_manager_breakpoint_manage()
	
	def transition_Root_main_breakpoint_manager_breakpoint_manage_2(self, parameters):
		del_breakpoint_id = parameters[0]
		self.exit_Root_main_breakpoint_manager_breakpoint_manage()
		self.breakpoints = [breakpoint for breakpoint in self.breakpoints if breakpoint.id != del_breakpoint_id]
		self.combo_step.setArenaChanged(self.Root_main_breakpoint_manager)
		self.enter_Root_main_breakpoint_manager_breakpoint_manage()
	
	def transition_Root_main_breakpoint_manager_breakpoint_manage_3(self, parameters):
		breakpoint_id = parameters[0]
		enabled = parameters[1]
		self.exit_Root_main_breakpoint_manager_breakpoint_manage()
		for breakpoint in self.breakpoints:
		    if breakpoint.id == breakpoint_id:
		        breakpoint.enabled = enabled
		        break
		self.combo_step.setArenaChanged(self.Root_main_breakpoint_manager)
		self.enter_Root_main_breakpoint_manager_breakpoint_manage()
	
	def generateCandidatesChildren_Root_main_reset(self):
		if self.current_state[self.Root_main_reset][0] == self.Root_main_reset_reset:
			return self.generateCandidates_Root_main_reset_reset()
		return False
	
	def generateCandidates_Root_main_reset(self):
		if not self.combo_step.isArenaChanged(self.Root_main_reset):
			return self.generateCandidatesChildren_Root_main_reset()
		else:
			return True
	
	def generateCandidatesCurrent_Root_main_reset_reset(self):
		enabled_events = self.getEnabledEvents()
		for e in enabled_events:
			if (e.name == "reset") and (e.port == "request"):
				self.small_step.addCandidate(self.transition_Root_main_reset_reset_1, e.parameters)
				return True
		return False
	
	def generateCandidates_Root_main_reset_reset(self):
		if not self.combo_step.isArenaChanged(self.Root_main_reset_reset):
			return self.generateCandidatesCurrent_Root_main_reset_reset()
		else:
			return True
	
	def transition_Root_main_reset_reset_1(self, parameters):
		self.exit_Root_main_reset_reset()
		for model in self.model.componentSet:
		    model.state = pickle.loads(self.save_model[model][1])
		    model.elapsed = self.save_model[model][0]
		    model.timeLast = (-model.elapsed, 1)
		    ta = model.timeAdvance()
		    model.timeNext = (model.timeLast[0] + ta, 1)
		self.simulation_time = (0.0, 0)
		self.model.scheduler.massReschedule(self.model.componentSet)
		
		# Reset trace file
		if self.trace_file is not None:
		    self.trace_file = open(self.trace_file.name, 'w')
		self.big_step.outputEvent(Event("all_states", "reply", [(0.0, 0), {m.getModelFullName(): (m.timeNext, m.state) for m in self.model.componentSet}]))
		self.combo_step.setArenaChanged(self.Root_main_reset)
		self.enter_Root_main_reset_reset()
	
	# Generate transition candidates for current small step
	def generateCandidates(self):
		self.generateCandidates_Root()

class ObjectManager(ObjectManagerBase):
	def __init__(self, controller):
		ObjectManagerBase.__init__(self, controller)
	
	def instantiate(self, class_name, construct_params):
		if class_name == "SCCDSimulator":
			instance = SCCDSimulator(self.controller, construct_params[0])
			instance.associations = {}
		return instance

class Controller(ThreadsControllerBase):
	def __init__(self, model, keep_running = None):
		if keep_running == None: keep_running = True
		ThreadsControllerBase.__init__(self, ObjectManager(self), keep_running)
		self.addInputPort("request")
		self.addOutputPort("reply")
		self.object_manager.createInstance("SCCDSimulator", [model])